(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "deep-eql", "kefir-test-utils"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("deep-eql"), require("kefir-test-utils"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.deepEql, global.kefirTestUtils);
    global.index = mod.exports;
  }
})(this, function (_exports, _deepEql, _kefirTestUtils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  _deepEql = _interopRequireDefault(_deepEql);
  _kefirTestUtils = _interopRequireDefault(_kefirTestUtils);

  function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

  function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

  function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

  function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

  function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

  var CHECK_OBS = 'CHECK_OBS';
  var CHECK_PROP = 'CHECK_PROP';
  var CHECK_STREAM = 'CHECK_STREAM';
  var CHECK_POOL = 'CHECK_POOL';
  var CHECK_ACTIVE = 'CHECK_ACTIVE';

  var noop = function noop() {};

  var _default = function _default(Kefir) {
    var _createTestHelpers = (0, _kefirTestUtils.default)(Kefir),
        END = _createTestHelpers.END,
        VALUE = _createTestHelpers.VALUE,
        ERROR = _createTestHelpers.ERROR,
        send = _createTestHelpers.send,
        value = _createTestHelpers.value,
        error = _createTestHelpers.error,
        end = _createTestHelpers.end,
        activate = _createTestHelpers.activate,
        deactivate = _createTestHelpers.deactivate,
        prop = _createTestHelpers.prop,
        stream = _createTestHelpers.stream,
        pool = _createTestHelpers.pool,
        shakeTimers = _createTestHelpers.shakeTimers,
        withFakeTime = _createTestHelpers.withFakeTime,
        logItem = _createTestHelpers.logItem,
        watch = _createTestHelpers.watch,
        watchWithTime = _createTestHelpers.watchWithTime;

    var plugin = function plugin(_ref, utils) {
      var Assertion = _ref.Assertion;
      Assertion.addChainableMethod('observable', function observableMethod() {
        var results = [true, 'should be valid', 'should not be valid'];
        var actual = utils.getActual(this, arguments);

        if (utils.flag(this, CHECK_OBS)) {
          if (!(actual instanceof Kefir.Observable)) {
            results = [false, "Expected ".concat(actual, " to be instance of Observable"), "Expected ".concat(actual, " to not be instance of Observable")];
          }

          if (utils.flag(this, CHECK_PROP) && !(actual instanceof Kefir.Property)) {
            results = [false, "Expected ".concat(actual, " to be instance of Property"), "Expected ".concat(actual, " to not be instance of Property")];
          }

          if (utils.flag(this, CHECK_STREAM) && !(actual instanceof Kefir.Stream)) {
            results = [false, '', ''];
          }

          if (utils.flag(this, CHECK_POOL) && !(actual instanceof Kefir.Pool)) {
            results = [false, '', ''];
          }

          if (utils.flag(this, CHECK_ACTIVE) && results[0]) {
            if (!actual._active) {
              results = [false, "Expected ".concat(actual.toString(), " to be active"), "Expected ".concat(actual.toString(), " to not be active")];
            }
          }
        }

        this.assert.apply(this, _toConsumableArray(results));
      }, function observableChaining() {
        utils.flag(this, CHECK_OBS, true);
      });
      Assertion.overwriteMethod('property', function property(_super) {
        return function checkIfPropety() {
          if (utils.flag(this, CHECK_OBS)) {
            utils.flag(this, CHECK_PROP, true);
            this.observable();
          } else {
            _super.apply(this, arguments);
          }
        };
      });
      Assertion.addMethod('stream', function stream() {
        utils.flag(this, CHECK_OBS, true);
        utils.flag(this, CHECK_STREAM, true);
        this.observable();
      });
      Assertion.addMethod('pool', function stream() {
        utils.flag(this, CHECK_OBS, true);
        utils.flag(this, CHECK_POOL, true);
        this.observable();
      });
      Assertion.addChainableMethod('active', function observableMethod() {
        this.observable();
      }, function observableChaining() {
        utils.flag(this, CHECK_OBS, true);
        utils.flag(this, CHECK_ACTIVE, true);
      });
      Assertion.addMethod('emit', function emitMethod(expected) {
        var cb = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : noop;
        var actual = utils.getActual(this, arguments);

        var _watch = watch(actual),
            log = _watch.log,
            unwatch = _watch.unwatch;

        cb();
        unwatch();
        this.assert((0, _deepEql.default)(log, expected), "Expected to emit #{exp}, actually emitted #{act}", "Expected to not emit #{exp}, actually emitted #{act}", expected, log);
      });
      Assertion.addMethod('emitInTime', function emitInTimeMethod(expected) {
        var cb = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : noop;

        var _ref2 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
            _ref2$timeLimit = _ref2.timeLimit,
            timeLimit = _ref2$timeLimit === void 0 ? 10000 : _ref2$timeLimit,
            _ref2$reverseSimultan = _ref2.reverseSimultaneous,
            reverseSimultaneous = _ref2$reverseSimultan === void 0 ? false : _ref2$reverseSimultan;

        var log = null;
        var actual = utils.getActual(this, arguments);
        withFakeTime(function (tick, clock) {
          log = watchWithTime(actual);
          cb(tick, clock);
          tick(timeLimit);
        }, reverseSimultaneous);
        this.assert((0, _deepEql.default)(log, expected), "Expected to emit #{exp}, actually emitted #{act}", "Expected to not emit #{exp}, actually emitted #{act}", expected, log);
      });
      Assertion.addMethod('flowErrors', function flowErrorsMethod() {
        var source = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : utils.getActual(this, arguments);
        var actual = utils.getActual(this, arguments);
        var expected = [error(-2), error(-3)];

        if (actual instanceof Kefir.Property) {
          activate(actual);
          send(source, [error(-1)]);
          deactivate(actual);
          expected.unshift(error(-1, {
            current: true
          }));
        }

        var _watch2 = watch(actual),
            log = _watch2.log,
            unwatch = _watch2.unwatch;

        send(source, [error(-2), error(-3)]);
        unwatch();
        this.assert((0, _deepEql.default)(log, expected), "Expected errors to flow (i.e. to emit #{exp}, actually emitted #{act})", "Expected errors not to flow (i.e. to emit [], actually emitted #{act})", expected, log);
      });
    };

    return {
      plugin: plugin,
      activate: activate,
      deactivate: deactivate,
      send: send,
      value: value,
      error: error,
      end: end,
      stream: stream,
      prop: prop,
      pool: pool
    };
  };

  _exports.default = _default;
});